// ZenLib::ZenTypes - To be independant of platform & compiler
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ZenTypes
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.3.2
// -------------
// Independancies of types
//               (u)int  (u)int  (u)int  (u)int  Float   Float
//                8bit   16bit   32bit   64bit   32bit   64bit
// Win32        :   X      X       X       X       X       X   
// Linux        :   X      X       X
//
// Platform detection
// __WIN32__, __UNIX__
//
// Easy Unicode Support
// 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// 0.3.2 (Zen@MediaArea.net, 2003-09-13)
// Unicode
//
// 0.3.1 (Zen@MediaArea.net, 2003-06-22)
// Add Float
//
// 0.3.0 (Zen@MediaArea.net, 2003-06-22)
// Platform definition
//
// 0.2.0 (Zen@MediaArea.net)
// Win32 64 bits
// Linux 8-32 bits
//
// 0.1.0 (Zen@MediaArea.net)
// Win32 8-32 bits
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#ifndef ZenTypesH
#define ZenTypesH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include <limits.h>
#include <tchar.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

//***************************************************************************
// Compiler bugs
//***************************************************************************

//MSVC6 : for(int t=0; t<10; ++t) { do something }; for(int t=0; t<10; ++t) { do something } 
#if _MSC_VER <= 1200
    #define for if(true)for
#endif

//***************************************************************************
// International
//***************************************************************************

//---------------------------------------------------------------------------
//Unicode
#if (defined(_UNICODE) || defined(UNICODE) || defined(__UNICODE__))
    #ifndef _UNICODE
        #define _UNICODE
    #endif
    #ifndef UNICODE
        #define UNICODE
    #endif
    #ifndef __UNICODE__
        #define __UNICODE__
    #endif
#endif

//---------------------------------------------------------------------------
//Char types
typedef _TCHAR Char;

//***************************************************************************
// Platforms
//***************************************************************************

//---------------------------------------------------------------------------
//Win32 (Win95, Win98, WinMe, WinNT, WinXP)
#if (defined(__NT__) || defined(_WIN32) || defined(WIN32)) && !defined(__WIN32__)
    #define __WIN32__
#endif

//Unix (Linux, HP, Sun, BeOS...)
#if (defined(__unix) || defined(__unix__) || \
      defined(____SVR4____) || defined(__LINUX__) || defined(__sgi) || \
      defined(__hpux) || defined(sun) || defined(__SUN__) || defined(_AIX) || \
      defined(__EMX__) || defined(__VMS) || defined(__BEOS__)) && !defined(__UNIX__)
#define __UNIX__
#endif

//Test of targets
#if defined(__WIN32__) && defined(__UNIX__)
    #error Two platforms???
#endif

#if !defined(__WIN32__) && !defined(__UNIX__)
    #pragma message No known platforms, assume default
#endif

//***************************************************************************
// Types
//***************************************************************************

//---------------------------------------------------------------------------
//int
typedef signed   int        ints;
typedef unsigned int        intu;

//---------------------------------------------------------------------------
//8-bit int
#if UCHAR_MAX==0xff
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 8
    typedef signed   char       int8s;
    typedef unsigned char       int8u;
#else
    #pragma message This machine has no 8-bit integertype?
#endif

//---------------------------------------------------------------------------
//16-bit int
#if UINT_MAX == 0xffff
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 16
    typedef signed   int        int16s;
    typedef unsigned int        int16u;
#elif USHRT_MAX == 0xffff
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 16
    typedef signed   short      int16s;
    typedef unsigned short      int16u;
#else
    #pragma message This machine has no 16-bit integertype?
#endif

//---------------------------------------------------------------------------
//32-bit int
#if UINT_MAX == 0xfffffffful
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 32
    typedef signed   int        int32s;
    typedef unsigned int        int32u;
#elif ULONG_MAX == 0xfffffffful
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 32
    typedef signed   long       int32s;
    typedef unsigned long       int32u;
#elif USHRT_MAX == 0xfffffffful
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 32
    typedef signed   short      int32s;
    typedef unsigned short      int32u;
#else
    #pragma message This machine has no 32-bit integer type?
#endif

//---------------------------------------------------------------------------
//64-bit int
#if defined(__MINGW32__) || defined(__CYGWIN32__)
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 64
    typedef signed   long long  int64s; //not tested!!!
    typedef unsigned long long  int64u; //not tested!!!
#elif defined(__WIN32__)
    #undef  MAXTYPE_INT
    #define MAXTYPE_INT 64
    typedef signed   __int64    int64s;
    typedef unsigned __int64    int64u;
#else
    #pragma message This machine has no 64-bit integer type?
#endif

//---------------------------------------------------------------------------
//32-bit float
//#if FLT_DIG >= 5 || _MSC_VER //Microsoft doesn't know FLT_DIG???
    #undef  MAXTYPE_FLOAT
    #define MAXTYPE_FLOAT 32
    typedef float                float32;
//#else
//    #pragma message This machine has no 32-bit float type?
//#endif

//---------------------------------------------------------------------------
//64-bit float
//#if DBL_DIG >= 15 || _MSC_VER //Microsoft doesn't know DBL_DIG???
    #undef  MAXTYPE_FLOAT
    #define MAXTYPE_FLOAT 64
    typedef double                float64;
//#else
//    #pragma message This machine has no 64-bit float type?
//#endif

} //namespace
#endif
